@@ -72,7 +72,6 @@ typedef std::vector<ex> exvector;
 typedef std::set<ex, ex_is_less> exset;
 typedef std::map<ex, ex, ex_is_less> exmap;
 using ex_int_map = std::map<GiNaC::ex, int, GiNaC::ex_is_less>;
-
 // Define this to enable some statistical output for comparisons and hashing
 #undef GINAC_COMPARE_STATISTICS
 
@@ -139,7 +138,6 @@ public:
 
 	basic(const archive_node &n, lst &sym_lst);
 	virtual void archive(archive_node &n) const;
-	static ex unarchive(const GiNaC::archive_node &n, GiNaC::lst &sym_lst);
 
 	class visitor {
 	public:
@@ -181,7 +179,6 @@ public: // only const functions please (may break reference counting)
 	virtual ex eval(int level = 0) const;
 	virtual ex evalf(int level = 0, PyObject* parent=nullptr) const;
 	virtual ex evalm() const;
-public:
 
 	// printing
 	virtual void print(const print_context & c, unsigned level = 0) const;
@@ -191,6 +188,10 @@ public:
 
 	// info
 	virtual bool info(unsigned inf) const;
+        virtual bool is_integer() const { return info(info_flags::integer); }
+        virtual bool is_real() const { return info(info_flags::real); }
+        virtual bool is_positive() const { return info(info_flags::positive); }
+        virtual bool is_negative() const { return info(info_flags::negative); }
 
 	// operand access
 	virtual size_t nops() const;
@@ -203,14 +204,13 @@ public:
 
 	// pattern matching
 	virtual bool has(const ex & other, unsigned options = 0) const;
-	virtual bool match(const ex & pattern, lst & repl_lst) const;
-protected:
+	virtual bool match(const ex & pattern, exmap& map) const;
 	virtual bool match_same_type(const basic & other) const;
 
 	virtual void do_print(const print_context & c, unsigned level) const;
 	virtual void do_print_tree(const print_tree & c, unsigned level) const;
-	virtual void do_print_python_repr(const print_python_repr & c, unsigned level) const;
-public:
+	virtual void do_print_python_repr(const print_python_repr & c,
+                        unsigned level) const;
 
 	// substitutions
 	virtual ex subs(const exmap & m, unsigned options = 0) const;
@@ -236,14 +236,14 @@ public:
 	virtual ex collect(const ex & s, bool distributed = false) const;
 
 	// differentiation and series expansion
-protected:
 	virtual ex derivative(const symbol & s) const;
-public:
-	virtual ex series(const relational & r, int order, unsigned options = 0) const;
+	virtual ex series(const relational & r, int order,
+                        unsigned options = 0) const;
         virtual void useries(flint_series_t& fp, int order) const {}
 
 	// rational functions
-	virtual ex normal(exmap & repl, exmap & rev_lookup, int level = 0, unsigned options = 0) const;
+	virtual ex normal(exmap & repl, exmap & rev_lookup, int level = 0,
+                        unsigned options = 0) const;
 	virtual ex to_rational(exmap & repl) const;
 	virtual ex to_polynomial(exmap & repl) const;
 
@@ -263,7 +263,6 @@ public:
 
 	virtual int compare(const basic & other) const;
 	// functions that should be called from class ex only
-protected:
 	virtual int compare_same_type(const basic & other) const;
 	virtual bool is_equal_same_type(const basic & other) const;
 
@@ -290,7 +289,7 @@ public:
         bool is_evaluated() const
             { return global_hold or ((flags & status_flags::evaluated) != 0u); }
 	const basic & hold() const;
-
+        void set_epseq_from(size_t i, ex e);
 	long gethash() const
 	{
 #ifdef GINAC_COMPARE_STATISTICS
@@ -313,14 +312,12 @@ public:
 	/** Clear some status_flags. */
 	const basic & clearflag(unsigned f) const {flags &= ~f; return *this;}
 
-protected:
 	void ensure_if_modifiable() const;
 #ifdef PYNAC_HAVE_LIBGIAC
         const giac::polynome to_polynome(ex_int_map& map, exvector& revmap);
 #endif
 
 	// member variables
-protected:
 	tinfo_t tinfo_key;                  ///< type info
 	mutable unsigned flags;             ///< of type status_flags
 	mutable long hashvalue=0;         ///< hash value
